

// LES FORMES

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*abstract*/ class Forme { 
  PShape s; // le contour de la forme
 
  protected Forme() { // constructeur
    s=createShape(); // creation de la forme
  }
  
  Forme(float... xys) { 
    s=createShape(); // creation de la forme
    vertex(xys);
  }
 
  final void vertex(float... xys) { int n; // methode utilitaire
    n=xys.length; // taille du tableau
    assert n%2==0; // test de parite 
    s.beginShape();
    for (int i=0; i<n; i+=2) s.vertex(xys[i],xys[i+1]); 
    s.endShape();
  }

  Forme translation(float x,float y) { s.applyMatrix(1,0,x, 0,1,y); return this; } // translation
  Forme symetrie() { s.applyMatrix(-1,0,9, 0,1,0); return this; } // symetrie
  Forme rotation() { s.applyMatrix(0,-1,9, 1,0,0); return this; } // rotation
  Forme rotation(int n) { for (int i=0; i<n; i++) rotation(); return this; } // rotations

  void dessiner(int c) { // dessin de la forme
    s.setFill(c); // remplissage de la forme avec la couleur c
    shape(s,0,0); // dessin de la forme
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class FormeDroit extends Forme { 
  FormeDroit() { vertex(0,3, 9,3, 9,6, 0,6); } // constructeur

  FormeDroit(int n) { vertex(0,3, 9*n,3, 9*n,6, 0,6); } // constructeur
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeArc45 extends Forme { 
  FormeArc45() { // constructeur (petit arc de 45°)     
    s.beginShape();
    s.vertex(0,3); s.quadraticVertex(4,3, 8,-1); s.vertex(10,1); s.quadraticVertex(5,6, 0,6); 
    s.endShape(CLOSE);
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeArc90 extends Forme { 
  FormeArc90(int n) { // constructeur (grand arc de 90° de rayon n)
    float nm=0.551784F; // nombre magique
    s.beginShape();
    s.vertex(0,3); s.bezierVertex((9*(n-1)+6)*nm, 3,   9*(n-1)+6, (9*n)-(9*(n-1)+6)*nm,   9*(n-1)+6, 9*(n-1)+9); // cubic
    s.vertex(9*(n-1)+3,9*(n-1)+9); s.bezierVertex(9*(n-1)+3, (9*n)-(9*(n-1)+3)*nm,   (9*(n-1)+3)*nm, 6,    0, 6); // cubic
    s.endShape();
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeBiais extends Forme { 
  FormeBiais() { // constructeur
    vertex(8,-1, 10,1, 1,10, -1,8);
  } 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeVide extends Forme { // forme pour la tjs (voie absente)
  FormeVide() { vertex(0,3, 4,3, 8,-1, 10,1, 5,6, 0,6); } // constructeur
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class FormeButoirDroit extends Forme { // partie voie du butoir
  FormeButoirDroit() { vertex(7,3, 0,3, 0,6, 7,6); } // constructeur
}

class FormeButoirBiais extends Forme { // partie voie du butoir
  FormeButoirBiais() { vertex(-1,8, 5,2, 7,4, 1,10); } // constructeur
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeTraverseDroit extends Forme { // traverse du butoir
  FormeTraverseDroit() { vertex(9,2, 7,2, 7,7, 9,7); } // constructeur
}

class FormeTraverseBiais extends Forme { // traverse du butoir
  //FormeTraverseBiais() { vertex(4,1, 6,-1, 10,3, 8,5); } // constructeur
  FormeTraverseBiais() { vertex(4,1, 5.75,-0.75, 9.75,3.25, 8,5); } // constructeur
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class FormeBiSens extends Forme {
  FormeBiSens() { // constructeur
   vertex(0,3, 1,3, 3,1.5, 3,3, 6,3, 6,1.5, 8,3, 9,3, 9,6, 8,6, 6,7.5, 6,6, 3,6, 3,7.5, 1,6, 0,6); 
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeSens extends Forme { 
  FormeSens() { // constructeur
    vertex(0,3, 3.5,3, 3.5,1.5, 5.5,3, 9,3, 9,6, 5.5,6, 3.5,7.5, 3.5,6, 0,6); 
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class FormeCible extends Forme { // cible pour les signaux (avec mat)
  FormeCible() { // constructeur
    vertex(3.4F,3, 3.4F,0.4F, 6,0.4F, 6,-1, 9,-1, 9,2, 6,2, 6,0.6F, 3.6F,0.6F, 3.6F,3); 
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeFeu extends Forme { // feu pour les signaux
  int feu;
  FormeFeu(int c) { feu=c; // constructeur
    s.beginShape(); // quatre arcs de 90°
    s.vertex(7.5F,-0.5F); 
    s.quadraticVertex(8.5F,-0.5F, 8.5F,0.5F); s.quadraticVertex(8.5F,1.5F, 7.5F,1.5F); 
    s.quadraticVertex(6.4F,1.5F, 6.5F,0.5F);  s.quadraticVertex(6.5F,-0.5F, 7.5F,-0.5F);
    s.endShape();
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////