
// LES FORMES

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class Forme { 
  PShape s; // le contour de la forme
 
  protected Forme() { // constructeur
    s=createShape(); // creation de la forme
  }
 
  final void vertex(float... xys) { int n; // methode utilitaire
    n=xys.length; // taille du tableau
    assert n%2==0; // test de parite 
    s.beginShape();
    for (int i=0; i<n; i+=2) s.vertex(xys[i],xys[i+1]); 
    s.endShape(CLOSE);
  }

  Forme translation(float x,float y) { s.applyMatrix(1,0,x, 0,1,y); return this; } // translation
  Forme symetrie() { s.applyMatrix(-1,0,9, 0,1,0); return this; } // symetrie
  Forme rotation() { s.applyMatrix(0,-1,9, 1,0,0); return this; } // rotation
  Forme rotation(int n) { for (int i=0; i<n; i++) rotation(); return this; } // rotations

  void dessiner(int c) { // dessin de la forme
    s.setFill(c); // remplissage de la forme avec la couleur c
    shape(s,0,0); // dessin de la forme
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class FormeSimple extends Forme { 
  FormeSimple(float... xys) { // constructeur
    vertex(xys);
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeDroit extends Forme { 
  FormeDroit() { // constructeur
    vertex(0,3, 9,3, 9,6, 0,6);
  }

  FormeDroit(int n) { // constructeur
    vertex(0,3, 9*n,3, 9*n,6, 0,6);
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeBiais extends Forme { 
  FormeBiais() { // constructeur
    vertex(8,-1, 10,1, 1,10, -1,8);
  } 
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class FormeArc45 extends Forme { 
  FormeArc45() { // constructeur (petit arc de 45°)     
    s.beginShape();
    s.vertex(0,3); s.quadraticVertex(4,3, 8,-1); s.vertex(10,1); s.quadraticVertex(5,6, 0,6); 
    s.endShape(CLOSE);
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class FormeArc90 extends Forme { 
  float nm=0.551784F; // nombre magique
  
  FormeArc90(int n) { // constructeur (grand arc de 90° de rayon n)
    s.beginShape();
    s.vertex(0,3); 
    s.bezierVertex((9*(n-1)+6)*nm, 3, 9*(n-1)+6, (9*n)-(9*(n-1)+6)*nm, 9*(n-1)+6, 9*(n-1)+9); // cubique
    s.vertex(9*(n-1)+3,9*(n-1)+9); 
    s.bezierVertex(9*(n-1)+3, (9*n)-(9*(n-1)+3)*nm, (9*(n-1)+3)*nm, 6, 0, 6); // cubique
    s.endShape();
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class FormeFeu extends Forme { // feu pour les signaux
  FormeFeu() { //constructeur
    s.beginShape(); // quatre arcs de 90°
    s.vertex(7.5,-0.5); 
    s.quadraticVertex(8.5,-0.5, 8.5,0.5); s.quadraticVertex(8.5,1.5, 7.5,1.5); 
    s.quadraticVertex(6.4,1.5, 6.5,0.5);  s.quadraticVertex(6.5,-0.5, 7.5,-0.5);
    s.endShape();
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class Retrait extends Forme { // ne pas heriter de forme ???  sym rot ???
  final float Rd=0.5,Ra=Rd/1.75; // les tailles des retraits (droit et angle)  Rd=0.5,Ra=Rd/1.75 enlever
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class RetraitDroit extends Retrait { // retrait est
  RetraitDroit() { // constructeur
    vertex(9,3, 9,6, 9-Rd,6, 9-Rd,3);
  }
  RetraitDroit(int n) { // constructeur
    vertex(9*n,3, 9*n,6, 9*n-Rd,6, 9*n-Rd,3);
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class RetraitAngle extends Retrait { // retrait nord est
  RetraitAngle() { // constructeur
    vertex(8,-1, 10,1, 10-Ra,1+Ra, 8-Ra,-1+Ra);
  }  
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////