
// LES PAVES

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class Pave {
  Forme[] formes; // les formes
  int couleur=GRIS; // couleur (par defaut) pour le pave
  int taille=1; // taile du pave (1 par defaut)
  ArrayList<Forme> retraits=new ArrayList<Forme>(); // les retraits
  
  Pave(Forme... fs) { formes=fs; } // constructeur
 
  Pave symetrie() { for (Forme f : formes) f.symetrie(); return this; } // symetrie
  Pave rotation() { for (Forme f : formes) f.rotation(); return this; } // rotation
  Pave rotation(int n) { for (int i=0; i<n; i++) rotation(); return this; } // rotations
  Pave translation(float x,float y) { for (Forme f : formes) f.translation(x,y); return this; } // translation
  
  Pave manoeuvrer() { manoeuvrer(false); return this; } // manoeuvre des aiguilles simples et signaux
  Pave manoeuvrer(boolean b) { return this; } // manoeuvre des aiguilles doubles (TJD TJS triple)
  
  void dessiner() { int l;  // dessin du pave
    l=formes.length; // le nombre de formes
    for (int i=0; i<l; i++) { // pour toutes les formes
      if (i==l-1) formes[i].dessiner(couleur); // la derniere forme (gris ou autre)
      else        formes[i].dessiner(GRIS_FONCE); // les autres formes (gris fonce)
    }
    for (Forme f : retraits) f.dessiner(GRIS_FOND); // dessin des retraits
  }
 
  void dessinRetraits() {for (Forme f : retraits) f.dessiner(GRIS_FOND); } // dessin des retraits

  final Pave retraitEst()   { retraits.add(new RetraitDroit()); return this; } 
  final Pave retraitSud()   { retraits.add(new RetraitDroit().rotation()); return this; }
  final Pave retraitOuest() { retraits.add(new RetraitDroit().rotation(2)); return this; } 
  final Pave retraitNord()  { retraits.add(new RetraitDroit().rotation(3)); return this; }

  final Pave retraitEst(int n)   { retraits.add(new RetraitDroit(n)); return this; } 
  final Pave retraitSud(int n)   { retraits.add(new RetraitDroit(n).rotation());  return this; }
  final Pave retraitOuest(int n) { retraits.add(new RetraitDroit(n).rotation(2)); return this; } 
  final Pave retraitNord(int n)  { retraits.add(new RetraitDroit(n).rotation(3)); return this; }

  final Pave retraitNordEst()   { retraits.add(new RetraitAngle()); return this; }
  final Pave retraitSudEst()    { retraits.add(new RetraitAngle().rotation()); return this; }
  final Pave retraitSudOuest()  { retraits.add(new RetraitAngle().rotation(2)); return this; }
  final Pave retraitNordOuest() { retraits.add(new RetraitAngle().rotation(3));  return this; }
  
  void colorer(int c) { couleur=c; } // coloration du pave (pour occupation et itineraires)
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveDroit extends Pave {
  PaveDroit() { super(new FormeDroit()); } // constructeur
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveDroitLong extends Pave { 
  PaveDroitLong(int n) { // constructeur
    super(new FormeSimple(0,3, 9*n,3, 9*n,6, 0,6));
    taille=n;
  } 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveBiais extends Pave {
  PaveBiais() { super(new FormeBiais()); } // constructeur
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveArc45 extends Pave {
  PaveArc45() { super(new FormeArc45()); } // constructeur
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveArc90Grand extends Pave {
  PaveArc90Grand(int n) { // constructeur
    super(new FormeArc90(n)); 
    taille=n;
  } 
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class PaveAiguilleSimple extends Pave { // regroupement des aiguilles simples
  PaveAiguilleSimple(Forme... ps) { super(ps); } // constructeur

  Pave manoeuvrer(boolean b) { Forme f; f=formes[0]; formes[0]=formes[1]; formes[1]=f; return this; } // permutation
}


//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleDroite extends PaveAiguilleSimple {
  PaveAiguilleDroite() { super(new FormeArc45(), // constructeur
                               new FormeDroit()); }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleBiaise extends PaveAiguilleSimple {
  PaveAiguilleBiaise() { super(new FormeArc45().rotation(2), // constructeur
                               new FormeBiais()); }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleSymetrique extends PaveAiguilleSimple {
  PaveAiguilleSymetrique() { super(new FormeArc45().symetrie().rotation(2), // constructeur
                                   new FormeArc45());
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class PaveAiguilleDouble extends Pave { // regroupement des aiguilles doubles
  PaveAiguilleDouble(Forme... ps) { super(ps); } // constructeur

  Pave manoeuvrer(boolean b) { Forme f; 
    if (b) { f=formes[0]; formes[0]=formes[2]; formes[2]=f; f=formes[1]; formes[1]=formes[3]; formes[3]=f; }  // permutations
    else   { f=formes[0]; formes[0]=formes[1]; formes[1]=f; f=formes[2]; formes[2]=formes[3]; formes[3]=f; }  // permutations
    return this; 
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveTJD extends PaveAiguilleDouble { // TJD
  PaveTJD() { 
    super(new FormeBiais(),
          new FormeArc45().rotation(2),
          new FormeArc45(),
          new FormeDroit()); 
  } 
}
  
//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveTJS extends PaveAiguilleDouble { // TJS    forme vide ??? nom ?

  PaveTJS() { 
    super(new FormeBiais(),
          new FormeSimple().rotation(2), 
          new FormeArc45(),
          new FormeDroit()); 
  }
}
 
//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveAiguilleTriple extends PaveAiguilleDouble { // modele peco asymetrique 
  boolean man=false; // position de la premiere aiguille 
  
  PaveAiguilleTriple() { // constructeur
    super(new FormeArc45(),
          new FormeArc45().symetrie().rotation(2),
          new FormeDroit()); 
  }

  Pave manoeuvrer(boolean b) { Forme f; 
    if (b) if (!man) { f=formes[0]; formes[0]=formes[2]; formes[2]=f; } // permutation 
           else      { f=formes[0]; formes[0]=formes[1]; formes[1]=f; } // permutation 
    else  { f=formes[1]; formes[1]=formes[2]; formes[2]=f; } // permutation 
    if (!b) man=!man; // memorisation de l'etat de la premiere aiguille
    return this; 
  } 
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class PaveCroisement extends Pave { // regroupement des croisements
  PaveCroisement(Forme... ps) { super(ps); } // constructeur

  void dessiner() { 
    formes[0].dessiner(GRIS); formes[1].dessiner(couleur);
    for (Forme f : retraits) f.dessiner(GRIS_FOND);
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveCroisementDroitDroit extends PaveCroisement { 
  PaveCroisementDroitDroit() { super(new FormeDroit().rotation(3), // constructeur
                                     new FormeDroit()); }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveCroisementDroitBiais extends PaveCroisement {
  PaveCroisementDroitBiais() { super(new FormeBiais(), // constructeur
                                     new FormeDroit()); }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveCroisementBiaisBiais extends PaveCroisement {
  PaveCroisementBiaisBiais() { super(new FormeBiais().rotation(), // constructeur
                                     new FormeBiais()); }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveButoirDroit extends Pave {
  PaveButoirDroit() { // constructeur
    super(new FormeSimple(9,2, 7,2, 7,7, 9,7), 
          new FormeSimple(7,3, 0,3, 0,6, 7,6)); 
  }
} 

class PaveButoirBiais extends Pave {
  PaveButoirBiais() { // constructeur
    super(new FormeSimple(4,1, 5.75,-0.75, 9.75,3.25, 8,5), 
          new FormeSimple(-1,8, 5,2, 7,4, 1,10)); 
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveBiSens extends Pave {
  PaveBiSens() { // constructeur
    super(new FormeSimple(0,3, 1,3, 3,1.5, 3,3, 6,3, 6,1.5, 8,3, 9,3, 9,6, 8,6, 6,7.5, 6,6, 3,6, 3,7.5, 1,6, 0,6)); 
  }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveSens extends Pave {
  PaveSens() { // constructeur
    super(new FormeSimple(0,3, 3.5,3, 3.5,1.5, 5.5,3, 9,3, 9,6, 5.5,6, 3.5,7.5, 3.5,6, 0,6)); 
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveSignal extends Pave { 
  Signal signal=null; // le signal
  int feu; // couleur du feu pour le tco

  PaveSignal(int f) { // constructeur
    super(new FormeSimple(3.4,3, 3.4,0.4, 6,0.4, 6,-1, 9,-1, 9,2, 6,2, 6,0.6, 3.6,0.6, 3.6,3),
          new FormeFeu(),
          new FormeDroit(),
          new RetraitDroit()
          ); 
    feu=f;
  }

  PaveSignal(Signal s) { // constructeur
    super(new FormeSimple(3.4,3, 3.4,0.4, 6,0.4, 6,-1, 9,-1, 9,2, 6,2, 6,0.6, 3.6,0.6, 3.6,3),
          new FormeFeu(),
          new FormeDroit(),
          new RetraitDroit()
          ); 
    signal=s; feu=s.feu.couleur; 
  }

  void dessiner() { 
    formes[0].dessiner(GRIS_FONCE); // cible et mat
    formes[1].dessiner(feu); // feu
    formes[2].dessiner(couleur); // voie
    formes[3].dessiner(GRIS_FOND); // retrait
    //for (Forme f : retraits) f.dessiner(GRIS_FOND); // dessin des retraits
    if (signal!=null && paveSignalSelectionne==this) signal.dessin();
  }
  
  Pave manoeuvrer(boolean b) { // pour manoeuvres par clic 
    if (signal!=null) { // avec signal
      if (signal.ferme()) { if (b) signal.ouvrirManoeuvre(); else signal.ouvrir(); } else signal.fermer(); 
      feu=signal.feu.couleur;
    } else { // sans signal
      switch (feu) {
        case ROUGE:  feu=VERT; break; 
        case VERT:   feu=ROUGE; break;
        case VIOLET: feu=BLANC; break;
        case BLANC:  feu=VIOLET ; break;
      }
    }
    return this;
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class PaveQuai extends Pave {
  int couleur=GRIS_FONCE;
  PaveQuai(int l)       { super(new FormeSimple(0,0, 9*l,0, 9*l,9, 0,9)); } // constructeur
  PaveQuai(int l,int c) { super(new FormeSimple(0,0, 9*l,0, 9*l,9, 0,9)); couleur=c; } // constructeur
  
  void dessiner() { formes[0].dessiner(couleur); }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class PaveTexte extends Pave {
  float dx=0,dy=0;
  int couleur=NOIR;
  float taille=2.5;
  String texte;
  
  PaveTexte(String s)               { texte=s; } // constructeurs ...
  PaveTexte(String s,float t)       { texte=s; taille=t; }
  PaveTexte(String s,int c)         { texte=s; couleur=c; }
  PaveTexte(String s,float t,int c) { texte=s; taille=t; couleur=c; }
  
  PaveTexte(String s,float dx,float dy)               { texte=s; this.dx=dx; this.dy=dy; }
  PaveTexte(String s,float t,float dx,float dy)       { texte=s; taille=t; this.dx=dx; this.dy=dy; }
  PaveTexte(String s,int c,float dx,float dy)         { texte=s; couleur=c; this.dx=dx; this.dy=dy; }
  PaveTexte(String s,float t,int c,float dx,float dy) { texte=s; taille=t; couleur=c; this.dx=dx; this.dy=dy; }
  
  void dessiner() { 
    fill(couleur); textAlign(CENTER,CENTER); textSize(taille*ZOOM); 
    pushMatrix();
    translate(4.5,4.5);
    scale(1.0/ZOOM);
    text(texte,dx*ZOOM,dy*ZOOM); 
    popMatrix();
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////