
// LES SIGNAUX

HashMap<String,PImage> cacheLumineux=new HashMap<String,PImage>(); // cache des signaux lumineux
HashMap<String,PImage> cacheMecaniques=new HashMap<String,PImage>(); // cache des signaux mecaniques

// LES SIGNAUX
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

enum Feu { // les feux possibles  deplacer ???
  ETEINT("E",GRIS_FONCE),
  VOIE_LIBRE("Vl",VERT),AVERTISSEMENT("A",JAUNE),SEMAPHORE("S",ROUGE,Info.FERME),SEMAPHORE_CLI("S",ROUGE,Info.FERME),
  CARRE("C",ROUGE,Info.FERME),MANOEUVRE("M",BLANC),CARRE_VIOLET("Cv",VIOLET,Info.FERME),
  RALENTISSEMENT("R",JAUNE),RALENTISSEMENT_CLI("R",JAUNE),
  RAPPEL_RALENTISSEMENT("RR",JAUNE,Info.R30),RAPPEL_RALENTISSEMENT_CLI("RR",JAUNE,Info.R60),
  COMBINE("ARR",JAUNE,Info.R30),COMBINE_CLI("ARR",JAUNE,Info.R60),
  BUTOIR("E",ROUGE),BUTOIR_MANOEUVRE("E",VIOLET),
  DISQUE("D",ROUGE,Info.FERME);

  enum Info {FERME,R30,R60};
  
  Info info=null;
  int couleur; // la couleur pour le tco
  String feu; // le nom du feu           
  private Feu(String s,int c)        { feu=s; couleur=c; info=null; } // constructeur
  private Feu(String s,int c,Info i) { feu=s; couleur=c; info=i; } // constructeur
  
  boolean ferme() { return info==Info.FERME; }
  boolean R30()   { return info==Info.R30; }
  boolean R60()   { return info==Info.R60; }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

abstract class Signal {
  Feu feu;
  PaveSignal pave; 
  String base; // base du nom de signal (C CR CRR S SR Cv cvB D ...)
  boolean estMecanique=false;
  //String nom;
  
  Signal(Feu f) { feu=f; }
  
  Signal mecanique() { estMecanique=true; return this; } // signal mecanique

  boolean ferme() { return feu.ferme(); } // methode de test
  boolean ouvert() { return !ferme(); } // methode de test

  void ouvrir() {} // ouverture manuelle 
  void ouvrirManoeuvre() {} // ouverture manuelle en manoeuvre
  void fermer() {} // fermeture manuelle 

  PImage icone(boolean h) { PImage i=null; String nom; // icone du signal (en fonction du feu)
    nom=base+"_"+feu.feu;
    if (estMecanique) { if ((i=cacheMecaniques.get(nom))==null) { i=loadImage("Mecaniques/"+nom+".gif"); cacheMecaniques.put(nom,i); }}
    else              { if ((i=cacheLumineux.get(nom))==null)   { i=loadImage("Lumineux/"+nom+".gif");   cacheLumineux.put(nom,i); }}
    if (h) i=halo(i); // ajout du halo
    return i;
  }
 
  void dessin() { PImage img; int i=0,l,h; float Z=2.5; int min,max; 
      pushMatrix(); 
      scale(1.0F/(ZOOM*Z)); // Z donne l'inverse de la taille du signal ("3" c'est plus petit)
      img=icone(true); 
      l=img.width; h=img.height; 
      if (this instanceof CarreVioletBas) { 
        while (i<l && img.get(i,h-1)!=#232323) i++; min=i; // recherche du mat (min)
        while (i<l && img.get(i,h-1)==#232323) i++; max=i; // recherche du mat (max)
      } else {
        while (i<l && img.get(i,h-1)!=#575757) i++; min=i; // recherche du mat (min)
        while (i<l && img.get(i,h-1)==#575757) i++; max=i; // recherche du mat (max)
      }
      image(img,(4.5*Z*ZOOM)-((max+min)/2.0),-h   +3*Z*ZOOM); // scale ici = 1/Z
      popMatrix();  
  }
  
  // fabrication du halo   tour de N pixels de plus en plus transparents

  int l,h; PImage halo;
  
  PImage halo(PImage image) { int i,x,y; color k; int N=16,NN=N+N; // ! N puissance de 2
    assert (N&(N-1))==0; // N puissance de 2
    //if (image==null) { erreur("Halo image nulle"); return null; }
    l=image.width;
    h=image.height;
    halo=createImage(l+NN,h+N,ARGB); // nouvelle image
    halo.loadPixels();
    for (i=0; i<(l+NN)*(h+N); i++) halo.pixels[i]=0; // tout transparent (simplifiable)
    halo.updatePixels();
    halo.loadPixels(); image.loadPixels();
    for (y=0; y<h; y++) for (x=0; x<l; x++) halo.pixels[(y*(l+NN))+N*(l+NN)+(x+N)]=image.pixels[y*l+x]; // copie
    halo.updatePixels();
    l+=NN; h+=N; //h+=NN;
    for (i=0; i<N; i++) { k=color(GRIS_FOND&0xFF,256-i*(256/N)); halo1(k); } // de plus en plus transparent
    return halo;
  }
  
  void halo1(color k) { int x,y; int c; // tour de 1 pixel
    halo.loadPixels();
    for (y=0; y<h; y++) for (x=0; x<l; x++) {
      c=get(x,y); 
      if (c!=0 && c!=k) { 
        if (get(x-1,y-1)==0) set(x-1,y-1,k);
        if (get(x  ,y-1)==0) set(x  ,y-1,k);
        if (get(x+1,y-1)==0) set(x+1,y-1,k);
        
        if (get(x-1,y  )==0) set(x-1,y  ,k);
        if (get(x+1,y  )==0) set(x+1,y  ,k);
        
        if (get(x-1,y+1)==0) set(x-1,y+1,k);
        if (get(x  ,y+1)==0) set(x  ,y+1,k);
        if (get(x+1,y+1)==0) set(x+1,y+1,k);
      }
    }
    halo.updatePixels();
  }
  
  int  get(int x,int y) { if (x<0 || y<0 || x>l-1 || y>h-1) return 255;  return halo.pixels[y*l+x]; }
  void set(int x,int y,int c) { if (x<0 || y<0 || x>l-1 || y>h-1) return;       halo.pixels[y*l+x]=c; }

}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

// Signal de bal

abstract class SignalBAL extends Signal {
  //boolean modePermanent=false;   
 
  SignalBAL(Feu f) { super(f); } // constructeur 
  
/*  
  void aubiner() {} // fermeture automatique par retrosignalisation
  void desaubiner() {} // ouverture automatique par retrosignalisation   

  void setPermanent(boolean b) { modePermanent=b; } // controle de l'ouverture permanente
  
  boolean cantonOccupe() { return true; } // test d'occupation du "canton" 
*/  
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class Carre extends Signal {
  Carre() { super(Feu.CARRE); base="C"; }

  void ouvrir()  { if (ouvert()) return; feu=Feu.VOIE_LIBRE; } // ouverture manuelle 
  void ouvrirManoeuvre()  { if (ouvert()) return; feu=Feu.MANOEUVRE; } // ouverture manuelle 
  void fermer()  { if (ferme()) return;  feu=Feu.CARRE; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class CarreAvertissement extends Signal {
  CarreAvertissement() { super(Feu.CARRE); base="CA"; }

  void ouvrir()  { if (ouvert()) return; feu=Feu.AVERTISSEMENT;  } // ouverture manuelle 
  void fermer()  { if (ferme()) return;  feu=Feu.CARRE; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class CarreRalentissement extends Carre {
  CarreRalentissement() { base="CR"; }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class CarreAvertissementRalentissement extends CarreAvertissement {
  CarreAvertissementRalentissement() { base="CAR"; }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class CarreRappelRalentissement extends Carre { 
  CarreRappelRalentissement() { base="CRR"; }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class CarreAvertissementRappelRalentissement extends CarreAvertissement {
  CarreAvertissementRappelRalentissement() { base="CARR"; }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class CarreViolet extends Signal {
  CarreViolet() { super(Feu.CARRE_VIOLET); base="Cv"; }

  void ouvrir()  {} // ouverture manuelle 
  void ouvrirManoeuvre()  { if (ouvert()) return; feu=Feu.MANOEUVRE; } // ouverture manuelle 
  void fermer()  { if (ferme()) return; feu=Feu.CARRE_VIOLET; } // fermeture manuelle 
}

class CarreVioletBas extends CarreViolet {
  CarreVioletBas() { base="Cvb"; }
}

class CarreVioletADroite extends CarreViolet { // carre implante a droite
  CarreVioletADroite() { base="Cvd"; }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class Avertissement extends Signal {
  Avertissement() { super(Feu.AVERTISSEMENT); base="A"; }

  void ouvrir()  { if (ouvert()) return; feu=Feu.VOIE_LIBRE; } // ouverture manuelle 
  void fermer()  { if (ferme()) return; feu=Feu.AVERTISSEMENT; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class AvertissementRalentissement extends Signal {
  AvertissementRalentissement() {super(Feu.AVERTISSEMENT); base="AR"; }

  void ouvrir()  { if (ouvert()) return; feu=Feu.RALENTISSEMENT; } // ouverture manuelle 
  void fermer()  { if (ferme()) return; feu=Feu.AVERTISSEMENT; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class Disque extends Signal {
  Disque() { super(Feu.DISQUE); base="D"; }

  void ouvrir()  { feu=Feu.VOIE_LIBRE; } // ouverture manuelle 
  void fermer()  { feu=Feu.DISQUE; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class DisqueAvertissement extends Signal {
  DisqueAvertissement() { super(Feu.DISQUE); base="DA"; }

  void ouvrir()  { feu=Feu.AVERTISSEMENT; } // ouverture manuelle 
  void fermer()  { feu=Feu.DISQUE; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class Semaphore extends Signal {
  Semaphore() { super(Feu.SEMAPHORE); base="S"; }

  void ouvrir()  { feu=Feu.VOIE_LIBRE; } // ouverture manuelle 
  void fermer()  { feu=Feu.SEMAPHORE; } // fermeture manuelle 
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CarreBAL extends Signal {
  CarreBAL() { super(Feu.CARRE); base="C"; }

  void ouvrir()  { feu=Feu.VOIE_LIBRE; } // ouverture manuelle 
  void fermer()  { feu=Feu.CARRE; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class CarreRalentissementBAL extends Signal {
  CarreRalentissementBAL() { super(Feu.CARRE); base="CR"; }

  void ouvrir()  { feu=Feu.RALENTISSEMENT; } // ouverture manuelle 
  void fermer()  { feu=Feu.CARRE; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class CarreRappelRalentissementBAL extends Signal {
  CarreRappelRalentissementBAL() { super(Feu.CARRE); base="CRR"; }

  void ouvrir()  { feu=Feu.RAPPEL_RALENTISSEMENT; } // ouverture manuelle 
  void fermer()  { feu=Feu.CARRE; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //


class SemaphoreBAL extends Signal {
  SemaphoreBAL() { super(Feu.VOIE_LIBRE); base="S"; }

  //void ouvrir()  { feu=Feu.VOIE_LIBRE; } // ouverture manuelle 
  //void fermer()  { feu=Feu.AVERTISSEMENT; } // fermeture manuelle 
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class SemaphoreAvertissementBAL extends Signal {
  SemaphoreAvertissementBAL() { super(Feu.AVERTISSEMENT); base="S"; }
}

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

class SemaphoreRalentissementBAL extends Signal {
  SemaphoreRalentissementBAL() { super(Feu.RALENTISSEMENT); base="SR"; }

  void ouvrir()  { feu=Feu.AVERTISSEMENT; } // ouverture manuelle 
  void fermer()  { feu=Feu.RALENTISSEMENT; } // fermeture manuelle 
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////